@@ -5,7 +5,7 @@ Tools to create and communicate with Micro Services
 """
 
 __author__ = "Chris Nasr"
-__copyright__ = "OuroborosCoding"
+__copyright__ = "Ouroboros Coding Inc."
 __version__ = "1.0.0"
 __email__ = "chris@ouroboroscoding.com"
 __created__ = "2018-11-11"
@@ -18,8 +18,8 @@ from time import sleep, time
 # Pip imports
 import requests
 
-# Framework imports
-from . import Errors, JSON, Sesh
+# Module imports
+from . import Errors, JSON, Session
 
 __mbVerbose = False
 """Verbose Flag"""
@@ -41,7 +41,7 @@ __funcToRequest = {
 }
 """Map functions to REST types"""
 
-def request(service, action, path, data, sesh=None, environ=None):
+def request(service, action, path, data):
 	"""Request
 
 	Method to convert REST requests into HTTP requests
@@ -50,9 +50,7 @@ def request(service, action, path, data, sesh=None, environ=None):
 		service (str): The service we are requesting data from
 		action (str): The action to take on the service
 		path (str): The path of the request
-		data (mixed): The data being sent with the request
-		sesh (Sesh._Session): The optional session to pass with the request
-		environ (dict): Info related to the request
+		data (dict): The request data: 'body', 'session', and 'enviroment'
 
 	Raises:
 		ServiceException
@@ -68,15 +66,15 @@ def request(service, action, path, data, sesh=None, environ=None):
 		if 'instance' in __mdRegistered[service]:
 
 			# If verbose requested
-			if __mbVerbose: print('%s: Calling %s.%s("%s", %s)' % (str(datetime.now()), service, action, path, JSON.encode(data, 2)))
+			if __mbVerbose: print('%s: Calling %s.%s("%s", %s)' % (str(datetime.now()), service, action, path, JSON.encode(data['body'], 2)))
 
 			# Directly call the action
 			oResponse = getattr(__mdRegistered[service]['instance'], action)(
-				path, data, sesh, environ
+				path, data
 			)
 
 			# If verbose requested
-			if __mbVerbose:	print('%s: Returning %s\n' % (str(datetime.now()), JSON.encode(oResponse.toDict(), 2)))
+			if __mbVerbose:	print('%s: Returning %s\n' % (str(datetime.now()), JSON.encode(oResponse.to_dict(), 2)))
 
 		# Else if the service is running elsewhere
 		else:
@@ -87,32 +85,32 @@ def request(service, action, path, data, sesh=None, environ=None):
 			# Generate the URL to reach the service
 			sURL = __mdRegistered[service]['url'] + path
 
-			# Convert the data to JSON
-			sData = JSON.encode(data)
+			# Convert the body to JSON
+			sBody = JSON.encode(data['body'])
 
 			# Create the headers
 			dHeaders = {
 				'Content-Type': 'application/json; charset=utf-8',
-				'Content-Length': str(len(sData))
+				'Content-Length': str(len(sBody))
 			}
 
 			# If we have a session, add the ID to the headers
-			if sesh:
-				dHeaders['Authorization'] = sesh.id()
+			if 'session' in data and data['session']:
+				dHeaders['Authorization'] = data['session'].id()
 
 			# Try to make the request and store the response
 			iAttempts = 0
 			while True:
 				iAttempts += 1
 				try:
-					oRes = __funcToRequest[action][0](sURL, data=sData, headers=dHeaders)
+					oRes = __funcToRequest[action][0](sURL, data=sBody, headers=dHeaders)
 
 					# If the request wasn't successful
 					if oRes.status_code != 200:
 
 						# If we got a 401
 						if oRes.status_code == 401:
-							return Response.fromJSON(oRes.content)
+							return Response.from_json(oRes.content)
 						else:
 							return Response(error=(Errors.SERVICE_STATUS, '%d: %s' % (oRes.status_code, oRes.content)))
 
@@ -139,7 +137,7 @@ def request(service, action, path, data, sesh=None, environ=None):
 					return Response(error=(Errors.SERVICE_UNREACHABLE, str(e)))
 
 			# Else turn the content into an Response and return it
-			oResponse = Response.fromJSON(oRes.text)
+			oResponse = Response.from_json(oRes.text)
 
 		# Return the Response of the request
 		return oResponse
@@ -148,7 +146,7 @@ def request(service, action, path, data, sesh=None, environ=None):
 	else:
 		raise ResponseException(error=(Errors.SERVICE_NOT_REGISTERED, service))
 
-def create(service, path, data, sesh=None, environ=None):
+def create(service, path, data):
 	"""Create
 
 	Make a POST request
@@ -156,16 +154,14 @@ def create(service, path, data, sesh=None, environ=None):
 	Arguments:
 		service (str): The service to call
 		path (str): The path on the service
-		data (mixed): The data to pass to the request
-		sesh {Sesh._Session}: The optional session to send with the request
-		environ (dict): Info related to the request
+		data (dict): The request data: 'body', 'session', and 'enviroment'
 
 	Returns:
 		Response
 	"""
-	return request(service, 'create', path, data, sesh, environ)
+	return request(service, 'create', path, data)
 
-def delete(service, path, data, sesh=None, environ=None):
+def delete(service, path, data):
 	"""Delete
 
 	Make a DELETE request
@@ -173,16 +169,14 @@ def delete(service, path, data, sesh=None, environ=None):
 	Arguments:
 		service (str): The service to call
 		path (str): The path on the service
-		data (mixed): The data to pass to the request
-		sesh {Sesh._Session}: The optional session to send with the request
-		environ (dict): Info related to the request
+		data (dict): The request data: 'body', 'session', and 'enviroment'
 
 	Returns:
 		Response
 	"""
-	return request(service, 'delete', path, data, sesh, environ)
+	return request(service, 'delete', path, data)
 
-def internalKey(key = None):
+def internal_key(key = None):
 	"""Internal Key
 
 	Generates or validates an internal key so services can communicate with
@@ -233,7 +227,7 @@ def internalKey(key = None):
 		except Exception:
 			return False
 
-def read(service, path, data, sesh=None, environ=None):
+def read(service, path, data):
 	"""Read
 
 	Make a GET request
@@ -241,14 +235,12 @@ def read(service, path, data, sesh=None, environ=None):
 	Arguments:
 		service (str): The service to call
 		path (str): The path on the service
-		data (mixed): The data to pass to the request
-		sesh {Sesh._Session}: The optional session to send with the request
-		environ (dict): Info related to the request
+		data (dict): The request data: 'body', 'session', and 'enviroment'
 
 	Returns:
 		Response
 	"""
-	return request(service, 'read', path, data, sesh, environ)
+	return request(service, 'read', path, data)
 
 def register(services, restconf, salt, internal=5):
 	"""Register
@@ -313,7 +305,7 @@ def register(services, restconf, salt, internal=5):
 		else:
 			raise ValueError('services.%s' % str(k))
 
-def update(service, path, data, sesh=None, environ=None):
+def update(service, path, data):
 	"""Update
 
 	Make a PUT request
@@ -321,14 +313,12 @@ def update(service, path, data, sesh=None, environ=None):
 	Arguments:
 		service (str): The service to call
 		path (str): The path on the service
-		data (mixed): The data to pass to the request
-		sesh {Sesh._Session}: The optional session to send with the request
-		environ (dict): Info related to the request
+		data (dict): The request data: 'body', 'session', and 'enviroment'
 
 	Returns:
 		Response
 	"""
-	return request(service, 'update', path, data, sesh, environ)
+	return request(service, 'update', path, data)
 
 def verbose(flag=True):
 	"""Verbose
@@ -385,15 +375,15 @@ class Response(object):
 
 			# If we got an int, it's a code with no message string
 			if isinstance(error, int):
-				self.error = {"code": error, "msg": ''}
+				self.error = {'code': error, 'msg': ''}
 
 			# If we got a string, it's a message with no code
 			elif isinstance(error, str):
-				self.error = {"code": 0, "msg": error}
+				self.error = {'code': 0, 'msg': error}
 
 			# If it's a tuple, 0 is a code, 1 is a message
 			elif isinstance(error, tuple):
-				self.error = {"code": error[0], "msg": error[1]}
+				self.error = {'code': error[0], 'msg': error[1]}
 
 			# If we got a dictionary, assume it's already right
 			elif isinstance(error, dict):
@@ -408,7 +398,7 @@ class Response(object):
 
 				# Else, try to pull out the code and message
 				else:
-					self.error = {"code": error.args[0], "msg": ''}
+					self.error = {'code': error.args[0], 'msg': ''}
 					if len(error.args) > 1: self.error['msg'] = error.args[1]
 
 			# Else, we got something invalid
@@ -446,7 +436,7 @@ class Response(object):
 		# Convert the dict and return it
 		return JSON.encode(dRet)
 
-	def dataExists(self):
+	def data_exists(self):
 		"""Data Exists
 
 		Returns True if there is data in the Response
@@ -457,7 +447,7 @@ class Response(object):
 		try: return self.data != None
 		except AttributeError: return False
 
-	def errorExists(self):
+	def error_exists(self):
 		"""Error Exists
 
 		Returns True if there is an error in the Response
@@ -469,7 +459,7 @@ class Response(object):
 		except AttributeError: return False
 
 	@classmethod
-	def fromDict(cls, val):
+	def from_dict(cls, val):
 		"""From Dict
 
 		Converts a dict back into an Response
@@ -500,7 +490,7 @@ class Response(object):
 		return o
 
 	@classmethod
-	def fromJSON(cls, val):
+	def from_json(cls, val):
 		"""From JSON
 
 		Tries to convert a string made from str() back into an Response
@@ -518,9 +508,9 @@ class Response(object):
 		except TypeError as e: raise ValueError('val', str(e))
 
 		# Return the fromDict result
-		return cls.fromDict(d)
+		return cls.from_dict(d)
 
-	def toDict(self):
+	def to_dict(self):
 		"""To Dict
 
 		Converts the Response into a dict
@@ -547,7 +537,7 @@ class Response(object):
 		# Return the dict
 		return dRet
 
-	def warningExists(self):
+	def warning_exists(self):
 		"""Warning Exists
 
 		Returns True if there is a warning in the Response
@@ -558,9 +548,6 @@ class Response(object):
 		try: return self.warning != None
 		except AttributeError: return False
 
-# Backwards compatibilty
-Effect = Response
-
 class Error(Response):
 	"""Error
 
@@ -615,9 +602,6 @@ class ResponseException(Exception):
 		else:
 			super().__init__(Response(data, error, warning))
 
-# Backwards compatibilty
-EffectException = ResponseException
-
 class Service(object):
 	"""Service
 
@@ -627,23 +611,22 @@ class Service(object):
 	__pathToDef = {}
 	"""Map of paths to function name"""
 
-	def create(self, path, data, sesh=None, environ=None):
+	def create(self, path, req):
 		"""Create
 
 		Create a new object
 
 		Arguments:
 			path (str): The path passed to the request
-			data (mixed): The data sent with the request
-			sesh (Sesh._Session): The session passed to the request
-			environ (dict): Info related to the request
+			req (dict): The data being sent with the request, including body,
+						session, and environment variables
 
 		Return:
 			Response
 		"""
 
 		# Generate the method name from the URI
-		sMethod = self.pathToMethod(path, '_create')
+		sMethod = self.path_to_method(path, '_create')
 
 		# Try to find the method
 		try:
@@ -658,40 +641,30 @@ class Service(object):
 			else:
 				raise
 
-		# Create the params
-		dParams = {"data": data}
-
-		# If we have a session
-		if sesh: dParams['sesh'] = sesh
-
-		# If we have environ
-		if environ: dParams['environ'] = environ
-
 		# Try to call the method
 		try:
-			return f(**dParams)
+			return f(req)
 
-			# Response thrown
+		# Response thrown
 		except ResponseException as e:
 			return e.args[0]
 
-	def delete(self, path, data, sesh=None, environ=None):
+	def delete(self, path, req):
 		"""Delete
 
 		Delete an existing object
 
 		Arguments:
 			path (str): The path passed to the request
-			data (mixed): The data sent with the request
-			sesh (Sesh._Session): The session passed to the request
-			environ (dict): Info related to the request
+			req (dict): The data being sent with the request, including body,
+						session, and environment variables
 
 		Return:
 			Response
 		"""
 
 		# Generate the method name from the URI
-		sMethod = self.pathToMethod(path, '_delete')
+		sMethod = self.path_to_method(path, '_delete')
 
 		# Try to find the method
 		try:
@@ -706,20 +679,11 @@ class Service(object):
 			else:
 				raise
 
-		# Create the params
-		dParams = {"data": data}
-
-		# If we have a session
-		if sesh: dParams['sesh'] = sesh
-
-		# If we have environ
-		if environ: dParams['environ'] = environ
-
 		# Try to call the method
 		try:
-			return f(**dParams)
+			return f(req)
 
-			# Response thrown
+		# Response thrown
 		except ResponseException as e:
 			return e.args[0]
 
@@ -753,23 +717,22 @@ class Service(object):
 		"""
 		raise NotImplementedError('Must implement the "install" method')
 
-	def read(self, path, data = {}, sesh=None, environ=None):
+	def read(self, path, req):
 		"""Read
 
 		Read an existing object
 
 		Arguments:
 			path (str): The path passed to the request
-			data (mixed): The data sent with the request
-			sesh (Sesh._Session): The session passed to the request
-			environ (dict): Info related to the request
+			req (dict): The data being sent with the request, including body,
+						session, and environment variables
 
 		Return:
 			Response
 		"""
 
 		# Generate the method name from the URI
-		sMethod = self.pathToMethod(path, '_read')
+		sMethod = self.path_to_method(path, '_read')
 
 		# Try to find the method
 		try:
@@ -784,40 +747,30 @@ class Service(object):
 			else:
 				raise
 
-		# Create the params
-		dParams = {"data": data}
-
-		# If we have a session
-		if sesh: dParams['sesh'] = sesh
-
-		# If we have environ
-		if environ: dParams['environ'] = environ
-
 		# Try to call the method
 		try:
-			return f(**dParams)
+			return f(req)
 
-			# Response thrown
+		# Response thrown
 		except ResponseException as e:
 			return e.args[0]
 
-	def update(self, path, data, sesh=None, environ=None):
+	def update(self, path, req):
 		"""Update
 
 		Update an existing object
 
 		Arguments:
 			path (str): The path passed to the request
-			data (mixed): The data sent with the request
-			sesh (Sesh._Session): The session passed to the request
-			environ (dict): Info related to the request
+			req (dict): The data being sent with the request, including body,
+						session, and environment variables
 
 		Return:
 			Response
 		"""
 
 		# Generate the method name from the URI
-		sMethod = self.pathToMethod(path, '_update')
+		sMethod = self.path_to_method(path, '_update')
 
 		# Try to find the method
 		try:
@@ -832,25 +785,16 @@ class Service(object):
 			else:
 				raise
 
-		# Create the params
-		dParams = {"data": data}
-
-		# If we have a session
-		if sesh: dParams['sesh'] = sesh
-
-		# If we have environ
-		if environ: dParams['environ'] = environ
-
 		# Try to call the method
 		try:
-			return f(**dParams)
+			return f(req)
 
-			# Response thrown
+		# Response thrown
 		except ResponseException as e:
 			return e.args[0]
 
 	@classmethod
-	def pathToMethod(cls, path, append=''):
+	def path_to_method(cls, path, append=''):
 		"""Path to Method
 
 		Takes a path and converts it to the standard naming for Service methods
@@ -875,11 +819,7 @@ class Service(object):
 			iLen = len(path)
 			i = 0
 			while i < iLen:
-				if(path[i] in ['/', '_', '-']):
-					i += 1
-					sRet += path[i].upper()
-				else:
-					sRet += path[i]
+				sRet += (path[i] in ['/', '-']) and '_' or path[i]
 				i += 1
 
 			# Store it for next time
